@@ -31,6 +31,8 @@ class HTTP:
     Pointer, go to http://tools.ietf.org/html/draft-pbryan-zyp-json-pointer-00.
     """
 
+    ROBOT_LIBRARY_SCOPE = 'TEST SUITE'
+
     # internal
 
     def __init__(self):
@@ -69,28 +71,32 @@ class HTTP:
     def _post_process_request(self):
 
         if self._response != None:
+            self.log_response_status('DEBUG')
             self.log_response_headers('DEBUG')
             self.log_response_body('DEBUG')
 
+        next_request_should = self._next_request_should
+
+        # prepare next request context, even if one of the following assertions
+        # fail
+        self._next_request_should = True
+        self._request_headers = {}
+        self._request_body = None
+
         # check flag set by "Next Request Should Succeed"
-        if self._next_request_should == True:
+        if next_request_should == True:
             assert int(self.response.status[0:3]) < 400, \
                'Request should have succeeded, but was "%s".' % \
                self.response.status
 
         # check flag set by "Next Request Should Not Succeed"
-        elif self._next_request_should == False:
+        elif next_request_should == False:
             assert int(self.response.status[0:3]) >= 400, \
                'Request should not have succeeded, but was "%s".' % \
                self.response.status
 
-        elif self._next_request_should:
-            self.response_status_code_should_equal(self._next_request_should)
-
-        # prepare next request context
-        self._next_request_should = True
-        self._request_headers = {}
-        self._request_body = None
+        elif next_request_should:
+            self.response_status_code_should_equal(next_request_should)
 
     @property
     def app(self):
@@ -141,6 +147,7 @@ class HTTP:
         path = self._path_from_url_or_path(url)
 
         self._pre_process_request()
+        logger.debug("Performing %s request on http://%s%s" % (verb, self.app.host, url,))
         self._response = self._app.request(path, {}, self._request_headers,
                 method=verb.upper(),)
         self._post_process_request()
@@ -153,6 +160,7 @@ class HTTP:
         """
         path = self._path_from_url_or_path(url)
         self._pre_process_request()
+        logger.debug("Performing HEAD request on http://%s%s" % (self.app.host, url,))
         self._response = self.app.head(path, {}, self._request_headers)
         self._post_process_request()
 
@@ -164,6 +172,7 @@ class HTTP:
         """
         path = self._path_from_url_or_path(url)
         self._pre_process_request()
+        logger.debug("Performing GET request on http://%s%s" % (self.app.host, url))
         self._response = self.app.get(path, {}, self._request_headers)
         self._post_process_request()
 
@@ -177,6 +186,7 @@ class HTTP:
         kwargs = {}
         if 'Content-Type' in self._request_headers:
             kwargs['content_type'] = self._request_headers['Content-Type']
+        logger.debug("Performing POST request on http://%s%s" % (self.app.host, url))
         self._pre_process_request()
         self._response = self.app.post(path, self._request_body or {}, self._request_headers, **kwargs)
         self._post_process_request()
@@ -192,6 +202,7 @@ class HTTP:
         if 'Content-Type' in self._request_headers:
             kwargs['content_type'] = self._request_headers['Content-Type']
         self._pre_process_request()
+        logger.debug("Performing PUT request on http://%s%s" % (self.app.host, url))
         self._response = self.app.put(path, self._request_body or {}, self._request_headers, **kwargs)
         self._post_process_request()
 
@@ -203,6 +214,7 @@ class HTTP:
         """
         path = self._path_from_url_or_path(url)
         self._pre_process_request()
+        logger.debug("Performing DELETE request on %s" % url)
         self._response = self.app.delete(path, {}, self._request_headers)
         self._post_process_request()
 
@@ -210,6 +222,14 @@ class HTTP:
         """
         Follows a HTTP redirect if the previous response status code was a 301 or 302.
         """
+        location = self.response.headers.get('location')
+
+        if location is None:
+            self.log_response_headers('INFO')
+            raise Exception("Can not follow a response without a location header.")
+
+        logger.debug("Following response, last response's Location header was %s" % location)
+
         self._response = self.response.follow()
 
 
@@ -241,22 +261,6 @@ class HTTP:
 
     # status code
 
-    def response_should_succeed(self):
-        """
-        *DEPRECATED*
-        Fails if the response status code of the previous request was >= 400
-        """
-        assert int(self.response.status[0:3]) < 400, \
-               'Response should have succeeded, but was "%s".' % self.response.status
-
-    def response_should_not_succeed(self):
-        """
-        *DEPRECATED*
-        Fails if the response status code of the previous request was < 400
-        """
-        assert int(self.response.status[0:3]) > 399, \
-               'Response should not have succeeded, but was "%s".' % self.response.status
-
     def response_status_code_should_equal(self, status_code):
         """
         Fails if the response status code of the previous request was not the
@@ -408,6 +412,15 @@ class HTTP:
         else:
             logger.debug("No response body received", log_level)
 
+    def log_response_status(self, log_level='INFO'):
+        """
+        Logs the response status line
+
+        Specify `log_level` (default: "INFO") to set the log level.
+        """
+
+        logger.write("Response status line: %s" % self.response.status, log_level)
+
     # json
 
     def should_be_valid_json(self, json_string):
